/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvimagecanvas.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvgui/qvimagecanvas.h>

QVImageCanvas::QVImageCanvas(const QString name, QWidget *parent):QVCanvas(parent), QVPropertyContainer(name), _low(0.0), _high(255.0)
	{
	setName("QVImageCanvas for "+name);
	setWindowTitle(getName());
	};

void QVImageCanvas::viewer()
	{
	qDebug() << "QVImageCanvas::viewer()";

	readInputProperties();

	QList<QString> imageNamesC1uChar = getPropertyListByType< QVImage<uChar,1> >();
	if (imageNamesC1uChar.size() == 1)
		{
		QVImage<uChar,1> img = getPropertyValue<QVImage<uChar,1> >(imageNamesC1uChar.at(0));
		getQVPainter()->drawQVImage(&img);
		}

	QList<QString> imageNamesC3uChar = getPropertyListByType< QVImage<uChar,3> >();
	if (imageNamesC3uChar.size() == 1)
		{
		QVImage<uChar,3> img = getPropertyValue<QVImage<uChar,3> >(imageNamesC3uChar.at(0));
		getQVPainter()->drawQVImage(&img);
		}

	QList<QString> imageNamesC1sFloat = getPropertyListByType< QVImage<sFloat,1> >();
	if (imageNamesC1sFloat.size() == 1)
		{
		QVImage<sFloat,1> img = getPropertyValue<QVImage<sFloat,1> >(imageNamesC1sFloat.at(0));
		getQVPainter()->drawQVImage(&img);
		}

	QList<QString> polylineListNames = getPropertyListByType< QList<QVPolyline> >();
	for (int i= 0; i< polylineListNames.size(); i++)
		{
		QList<QVPolyline> contours = getPropertyValue< QList<QVPolyline> >(polylineListNames.at(i));
		draw(contours);
		}

	qDebug() << "QVImageCanvas::viewer() <~ return";
	}

void QVImageCanvas::linkProperty(QVWorker &worker, const QString propertyName)
	{
	addPropertyFromQVariant(propertyName, inputFlag, worker.getPropertyQVariantValue(propertyName), worker.getPropertyInfo(propertyName));
	worker.linkProperty(propertyName, this, propertyName ,QVWorker::AsynchronousLink);
	QObject::connect(&worker, SIGNAL(endIteration()), this, SLOT(refreshImageArea()));
	show();
	}

void QVImageCanvas::setLowHigh(float low,float high)
	{
	_low = low;
	_high = high;
	}

void QVImageCanvas::draw(const QList<QVPolyline> &contours)
	{
	for (int i = 0; i< contours.size(); i++)
		{
		QVPolyline polyline = contours.at(i);
		getQVPainter()->setPen(QColor(polyline.direction?Qt::red:Qt::blue));
		//getQVPainter()->drawText(QRect(polyline.at(0),polyline.at(0)), Qt::AlignCenter|Qt::TextDontClip, QString("%1").arg(polyline.size()));
		getQVPainter()->drawArc(QRect(polyline.at(0),polyline.at(0)),0,16*360);
		for (int j = 0; j< polyline.size()-1; j++)
			{
			getQVPainter()->drawLine(polyline.at(j)+QPointF(0.5,0.5),polyline.at(j+1)+QPointF(0.5,0.5)); // +0.5 to paint in the center of the pixel.
			}

		// Paint last point as an asterisc.
		QPoint last = polyline.last();
		getQVPainter()->drawLine(last + QPointF(0.72500,   0.88971), last + QPointF(0.27500,   0.11029));
		getQVPainter()->drawLine(last + QPointF(0.88971,   0.72500), last + QPointF(0.11029,   0.27500));
		getQVPainter()->drawLine(last + QPointF(0.88971,   0.27500), last + QPointF(0.11029,   0.72500));
		getQVPainter()->drawLine(last + QPointF(0.72500,   0.11029), last + QPointF(0.27500,   0.88971));


		if (polyline.closed) getQVPainter()->drawLine(polyline.last()+QPointF(0.5,0.5),polyline.at(0)+QPointF(0.5,0.5));
		}
	}

